// deno-lint-ignore-file no-explicit-any
import HttpError from "../HttpError.ts";
import { makeParameterModifier } from "../Router.ts";

export type PrimitiveSchema = 'string' | 'number' | 'boolean' | 'object';
export type OptionalSchema = `${PrimitiveSchema}?`;
export type Schema = 'any' | PrimitiveSchema | OptionalSchema | Schema[] | ({ $optional?: boolean; } & { [key: string]: Schema });

export default function schema(desc: Schema) {
    function stringify(desc: Schema): string {
        if (typeof desc === 'string') return desc;
        if (desc instanceof Array) return desc.map(stringify).join(' | ');

        let res = '{ ';

        for (const key in desc) {
            if (key === '$optional') continue;
            if (res != '{ ') res += ', ';
            res += key + ': ';
            res += stringify(desc[key]);
        }
        res += '}';
        if (desc.$optional) res += '?';

        return res;
    }
    function test(path: string[], val: unknown, desc: Schema) {
        const _path = path.join('.');
        if (desc === 'any') return;
        if (typeof desc === 'string') {
            let type: string = desc;
            const opt = desc.endsWith('?');

            if (opt) type = type.substring(0, desc.length - 1);
            if (opt && val === undefined) return;

            if (typeof val as string !== type) throw new HttpError(`${_path}: Expected a ${type}, got ${typeof val} instead.`);
        }
        else if (desc instanceof Array) {
            for (const type of desc) {
                try {
                    test(path, val, type);
                    return;
                }
                catch { /**/ }
            }
            throw new HttpError(`${_path}: Expected a ${stringify(desc)}, got ${typeof val} instead.`);
        }
        else {
            if (desc.$optional && val === undefined) return;
            if (typeof val !== 'object' || val === null) throw new HttpError(`${_path}: Expected an object, got ${typeof val} instead.`);

            for (const key in desc) {
                if (key === '$optional') continue;
                test([ ...path, key ], (val as any)[key], desc[key]);
            }
        }
    }

    return makeParameterModifier((_req, val, name) => (test([name], val, desc), val));
}
